#version 100

precision highp float;

bool isnan(float x)
{
  return !(x == x);
}

bool isinf(float x)
{
  return x == 1.0/0.0 || x == -1.0/0.0;
}

float tanh(float x)
{
  float e = exp(2.0 * x);
  return (e - 1.0) / (e + 1.0);
}

#define MAXCOUNT 512

const float er = 1.0e10;

uniform bool julia;
uniform int count;
uniform vec2 inflection[MAXCOUNT];

varying vec2 texcoord;

vec2 cMul(vec2 a, vec2 b)
{
  return vec2(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x);
}

vec2 cSqr(vec2 a)
{
  return cMul(a, a);
}

vec4 cMul(vec4 a, vec4 b)
{
  return vec4(cMul(a.xy, b.xy), cMul(a.xy, b.zw) + cMul(a.zw, b.xy));
}

vec4 cSqr(vec4 a)
{
  return cMul(a, a);
}

void main()
{
  float px = length(vec4(dFdx(texcoord), dFdy(texcoord)));
  vec4 c = vec4(texcoord, px, 0.0);
  for (int i = 0; i < count && i < MAXCOUNT; ++i)
  {
    vec4 f = vec4(inflection[i], 0.0, 0.0);
    vec4 d = c - f;
    c = cSqr(d) + f;
  }
  int n = 0;
  vec4 z = julia && count != 0 ? c : vec4(0.0, 0.0, 0.0, 0.0);
       c = julia && count != 0 ? vec4(inflection[count - 1], 0.0, 0.0) : c;
  for (n = 0; n < 1024; ++n)
  {
    if (dot(z.xy, z.xy) >= er)
      break;
    z = cSqr(z) + c;
  }
  if (dot(z.xy, z.xy) < er)
  {
    gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
    return;
  }
  float r = length(z.xy);
  float dr = length(z.zw);
  float de = 2.0 * r * log(r) / dr;
  float g = tanh(clamp(de, 0.0, 4.0));
  if (isnan(de) || isinf(de) || isnan(g) || isinf(g))
    g = 1.0;
  gl_FragColor = vec4(vec3(g), 1.0);
}
